@@ -4,8 +4,8 @@
 #include "npl.h"
 
 
-typedef void (*tpred)(char *, int(*pred)(int, int));
-typedef void (*tuser)(char *, int(*user)(int, int), int weight, int spec);
+typedef void (*tpred)(char *, int (*pred)(int, int));
+typedef void (*tuser)(char *, int (*user)(int, int), int weight, int spec);
 
 
 char *get_name(int x)
@@ -13,7 +13,7 @@ char *get_name(int x)
     return (GET_NAME(x));
 }
 
-
+// cppcheck-suppress resourceLeak
 void dynamic_link(int x)
 {
     char str[256] = { "./" };
@@ -38,8 +38,9 @@ void dynamic_link(int x)
 	strcat(str, GET_NAME(x));
 
     hmod = dlopen(str, RTLD_LAZY);
-    if (hmod == NULL)
+    if (hmod == NULL) {
 	exception(SYSTEM_ERR, makestr("load"), x, 0);
+    }
 
     init_f0 = dlsym(hmod, "init0");
     init_f1 = dlsym(hmod, "init1");
@@ -154,6 +155,7 @@ void dynamic_link(int x)
     init_f3(RIGHTSHIFT_IDX, (tpred) f_rightshift);
     init_f3(LOGICALAND_IDX, (tpred) f_logicaland);
     init_f3(LOGICALOR_IDX, (tpred) f_logicalor);
+    init_f3(XOR_IDX, (tpred) f_xor);
     init_f3(ROUND_IDX, (tpred) f_round);
     init_f3(DIV_IDX, (tpred) f_div);
     init_f3(CALL_IDX, (tpred) call);
